import 'package:flutter/material.dart';

import 'settings_service.dart';

/// A class that many Widgets can interact with to read user settings, update
/// user settings, or listen to user settings changes.
///
/// Controllers glue Data Services to Flutter Widgets. The SettingsController
/// uses the SettingsService to store and retrieve user settings.
class SettingsController with ChangeNotifier {
  SettingsController(
    this._settingsService,
  )   : _stringOpts = {},
        _boolOpts = {},
        _intOpts = {};

  final SettingsService _settingsService;

  Future<void> loadSettings() async {
    _themeMode = await _settingsService.themeMode();
    _stringOpts["protocol"] = await _settingsService.get<String>("protocol");
    _stringOpts["hostname"] = await _settingsService.get("hostname");
    _intOpts["port"] = await _settingsService.get("port");
    _stringOpts["username"] = await _settingsService.get("username");
    _stringOpts["password"] = await _settingsService.get("password");
    _boolOpts["lockApp"] = await _settingsService.get("lockApp");
    // Important! Inform listeners a change has occurred.
    notifyListeners();
  }

  late ThemeMode _themeMode;
  ThemeMode get themeMode => _themeMode;
  final Map<String, String?> _stringOpts;
  final Map<String, bool?> _boolOpts;
  final Map<String, int?> _intOpts;

  Future<void> set<T>(String key, T? value) async {
    if (value == null) return;
    await _settingsService.set(key, value);
    notifyListeners();
  }

  T get<T>(String key, T default_) {
    if (T is String) {
      return _stringOpts.containsKey(key) ? _stringOpts[key] as T : default_;
    } else if (T is bool) {
      return _boolOpts.containsKey(key) ? _boolOpts[key] as T : default_;
    } else if (T is int) {
      return _intOpts.containsKey(key) ? _intOpts[key] as T : default_;
    } else {
      throw Exception("this type is not implemented");
    }
  }

  /// Update and persist the ThemeMode based on the user's selection.
  Future<void> updateThemeMode(ThemeMode? newThemeMode) async {
    if (newThemeMode == null) return;

    // Do not perform any work if new and old ThemeMode are identical
    if (newThemeMode == _themeMode) return;

    // Otherwise, store the new ThemeMode in memory
    _themeMode = newThemeMode;

    // Important! Inform listeners a change has occurred.
    notifyListeners();

    // Persist the changes to a local database or the internet using the
    // SettingService.
    await _settingsService.updateThemeMode(newThemeMode);
  }
}
