import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

/// A service that stores and retrieves user settings.
///
/// By default, this class does not persist user settings. If you'd like to
/// persist the user settings locally, use the shared_preferences package. If
/// you'd like to store settings on a web server, use the http package.
class SettingsService {
  /// Loads the User's preferred ThemeMode from local or remote storage.
  Future<ThemeMode> themeMode() async {
    final SharedPreferences prefs = await SharedPreferences.getInstance();
    final themeIdx = prefs.getInt("theme");
    if (themeIdx == null) return ThemeMode.system;

    return ThemeMode.values[themeIdx];
  }

  /// Persists the user's preferred ThemeMode to local or remote storage.
  Future<void> updateThemeMode(ThemeMode theme) async {
    final SharedPreferences prefs = await SharedPreferences.getInstance();
    prefs.setInt("theme", theme.index);
  }

  Future<T?> get<T>(String key) async {
    final SharedPreferences prefs = await SharedPreferences.getInstance();
    T? res;

    if (T == String) {
      res = prefs.getString(key) as T?;
    } else if (T == bool) {
      res = prefs.getBool(key) as T?;
    } else if (T == int) {
      res = prefs.getInt(key) as T?;
    } else {
      res = null as T?;
    }
    return res;
  }

  Future<void> set<T>(String k, T v) async {
    final SharedPreferences prefs = await SharedPreferences.getInstance();
    if (T is String) {
      await prefs.setString(k, v as String);
    } else if (T is bool) {
      await prefs.setBool(k, v as bool);
    } else if (T is int) {
      await prefs.setInt(k, v as int);
    }
  }
}
